require 'paths'
require 'pp'

class Configuration

  attr_reader :full_traversal
  
  def initialize(browser, &block)
    @browser = browser
    @full_traversal = [:beginning]
    instance_eval(&block)
  end

  def we_have_a_visit_record_with(fields)
    data_entry(:add_visit_page, :case_display_page, fields)
  end

  def we_have_a_visit_record
    we_have_a_visit_record_with({})
  end

  def we_have_an_audit_record_with(fields)
    needs(:add_visit_page)
    data_entry(:add_audit_page, :case_display_page, fields)
  end

  def we_have_an_audit_record
    we_have_an_audit_record_with({})
  end

  # Note that this assumes that getting to the page means that whatever
  # data it creates has indeed been created.
  def needs(page)
    unless @full_traversal.any? { | t | t.class != Symbol && t.destination == page }
      move_to(page)
    end
  end

  def we_are_at(destination)
    path = Path.between(@full_traversal.last, destination)
    @full_traversal += path.path
  end
  alias_method(:be_at, :we_are_at)
  alias_method(:move_to, :we_are_at)

  def establish
    @full_traversal.each { | node |
      if node.respond_to?(:implement_through)
        node.implement_through(@browser)
      end
    }
  end

  private

  def data_entry(entry_page, post_submit_page, fields)
    be_at(entry_page)
    move_to(post_submit_page)
    fields.each_pair { | field, value | replace_default_with(field, value) }
  end


  # Hackish: replace on most recent transition that has that field.
  # Plus go wild with that ol' functional style. Whee!
  #
  # Kludge: this is terribly unsafe if more than one transition can have
  # a particular field. 
  def replace_default_with(field, value)
    @full_traversal.reverse.reject { | t |
      t.is_a?(Symbol)
    }.find { | t |
      t.has_field?(field)
    }.set_field(field, value)
  end
    
end
